# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: control/control.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x63ontrol/control.proto\x12\x18longbridgeapp.control.v1\"\xe4\x01\n\x05\x43lose\x12\x38\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.longbridgeapp.control.v1.Close.CodeR\x04\x63ode\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reason\"\x88\x01\n\x04\x43ode\x12\x14\n\x10HeartbeatTimeout\x10\x00\x12\x0f\n\x0bServerError\x10\x01\x12\x12\n\x0eServerShutdown\x10\x02\x12\x0f\n\x0bUnpackError\x10\x03\x12\r\n\tAuthError\x10\x04\x12\x0f\n\x0bSessExpired\x10\x05\x12\x14\n\x10\x43onnectDuplicate\x10\x06\"b\n\tHeartbeat\x12\x1c\n\ttimestamp\x18\x01 \x01(\x03R\ttimestamp\x12&\n\x0cheartbeat_id\x18\x02 \x01(\x05H\x00R\x0bheartbeatId\x88\x01\x01\x42\x0f\n\r_heartbeat_id\"#\n\x0b\x41uthRequest\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token\"u\n\x0c\x41uthResponse\x12\x1d\n\nsession_id\x18\x01 \x01(\tR\tsessionId\x12\x18\n\x07\x65xpires\x18\x02 \x01(\x03R\x07\x65xpires\x12\x14\n\x05limit\x18\x03 \x01(\rR\x05limit\x12\x16\n\x06online\x18\x04 \x01(\rR\x06online\"1\n\x10ReconnectRequest\x12\x1d\n\nsession_id\x18\x01 \x01(\tR\tsessionId\"z\n\x11ReconnectResponse\x12\x1d\n\nsession_id\x18\x01 \x01(\tR\tsessionId\x12\x18\n\x07\x65xpires\x18\x02 \x01(\x03R\x07\x65xpires\x12\x14\n\x05limit\x18\x03 \x01(\rR\x05limit\x12\x16\n\x06online\x18\x04 \x01(\rR\x06online*L\n\x07\x43ommand\x12\r\n\tCMD_CLOSE\x10\x00\x12\x11\n\rCMD_HEARTBEAT\x10\x01\x12\x0c\n\x08\x43MD_AUTH\x10\x02\x12\x11\n\rCMD_RECONNECT\x10\x03\x42\xf3\x01\n\x1c\x63om.longbridgeapp.control.v1B\x0c\x43ontrolProtoP\x01ZCgithub.com/longbridgeapp/openapi-protobufs/gen/go/control;controlv1\xa2\x02\x03LCX\xaa\x02\x18Longbridgeapp.Control.V1\xca\x02\x18Longbridgeapp\\Control\\V1\xe2\x02$Longbridgeapp\\Control\\V1\\GPBMetadata\xea\x02\x1aLongbridgeapp::Control::V1b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'control.control_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.longbridgeapp.control.v1B\014ControlProtoP\001ZCgithub.com/longbridgeapp/openapi-protobufs/gen/go/control;controlv1\242\002\003LCX\252\002\030Longbridgeapp.Control.V1\312\002\030Longbridgeapp\\Control\\V1\342\002$Longbridgeapp\\Control\\V1\\GPBMetadata\352\002\032Longbridgeapp::Control::V1'
  _COMMAND._serialized_start=713
  _COMMAND._serialized_end=789
  _CLOSE._serialized_start=52
  _CLOSE._serialized_end=280
  _CLOSE_CODE._serialized_start=144
  _CLOSE_CODE._serialized_end=280
  _HEARTBEAT._serialized_start=282
  _HEARTBEAT._serialized_end=380
  _AUTHREQUEST._serialized_start=382
  _AUTHREQUEST._serialized_end=417
  _AUTHRESPONSE._serialized_start=419
  _AUTHRESPONSE._serialized_end=536
  _RECONNECTREQUEST._serialized_start=538
  _RECONNECTREQUEST._serialized_end=587
  _RECONNECTRESPONSE._serialized_start=589
  _RECONNECTRESPONSE._serialized_end=711
# @@protoc_insertion_point(module_scope)
